Class: Linguist::Language

Inherits:
Object
  • Object
show all
Defined in:
lib/linguist/language.rb

Overview

Language names that are recognizable by GitHub. Defined languages can be highlighted, searched and listed under the Top Languages page.

Languages are defined in ‘lib/linguist/languages.yml`.

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(attributes = {}) ⇒ Language

Internal: Initialize a new Language

attributes - A hash of attributes



266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
# File 'lib/linguist/language.rb', line 266

def initialize(attributes = {})
  # @name is required
  @name = attributes[:name] || raise(ArgumentError, "missing name")

  @fs_name = attributes[:fs_name]

  # Set type
  @type = attributes[:type] ? attributes[:type].to_sym : nil
  if @type && !get_types.include?(@type)
    raise ArgumentError, "invalid type: #{@type}"
  end

  @color = attributes[:color]

  # Set aliases
  @aliases = [default_alias] + (attributes[:aliases] || [])

  @tm_scope = attributes[:tm_scope] || 'none'
  @ace_mode = attributes[:ace_mode]
  @codemirror_mode = attributes[:codemirror_mode]
  @codemirror_mime_type = attributes[:codemirror_mime_type]
  @wrap = attributes[:wrap] || false

  # Set the language_id
  @language_id = attributes[:language_id]

  # Set extensions or default to [].
  @extensions   = attributes[:extensions]   || []
  @interpreters = attributes[:interpreters] || []
  @filenames    = attributes[:filenames]    || []

  # Set popular flag
  @popular    = attributes.key?(:popular)    ? attributes[:popular]    : false

  # If group name is set, save the name so we can lazy load it later
  if attributes[:group_name]
    @group_name = attributes[:group_name]

  # Otherwise we can set it to self now
  else
    @group_name = self.name
  end
end

Instance Attribute Details

#ace_modeObject (readonly)

Public: Get Ace mode

Examples

# => "text"
# => "javascript"
# => "c_cpp"

Returns a String name or nil



375
376
377
# File 'lib/linguist/language.rb', line 375

def ace_mode
  @ace_mode
end

#aliasesObject (readonly)

Public: Get aliases

Examples

Language['C++'].aliases
# => ["cpp"]

Returns an Array of String names



348
349
350
# File 'lib/linguist/language.rb', line 348

def aliases
  @aliases
end

#codemirror_mime_typeObject (readonly)

Public: Get CodeMirror MIME type mode

Examples

# => "nil"
# => "text/x-javascript"
# => "text/x-csrc"

Returns a String name or nil



400
401
402
# File 'lib/linguist/language.rb', line 400

def codemirror_mime_type
  @codemirror_mime_type
end

#codemirror_modeObject (readonly)

Public: Get CodeMirror mode

Maps to a directory in the ‘mode/` source code.

https://github.com/codemirror/CodeMirror/tree/master/mode

Examples

# => "nil"
# => "javascript"
# => "clike"

Returns a String name or nil



389
390
391
# File 'lib/linguist/language.rb', line 389

def codemirror_mode
  @codemirror_mode
end

#colorObject (readonly)

Public: Get color.

Returns a hex color String.



338
339
340
# File 'lib/linguist/language.rb', line 338

def color
  @color
end

#extensionsObject (readonly)

Public: Get extensions

Examples

# => ['.rb', '.rake', ...]

Returns the extensions Array



414
415
416
# File 'lib/linguist/language.rb', line 414

def extensions
  @extensions
end

#filenamesObject (readonly)

Public: Get filenames

Examples

# => ['Rakefile', ...]

Returns the extensions Array



432
433
434
# File 'lib/linguist/language.rb', line 432

def filenames
  @filenames
end

#fs_nameObject (readonly)

Public:



328
329
330
# File 'lib/linguist/language.rb', line 328

def fs_name
  @fs_name
end

#interpretersObject (readonly)

Public: Get interpreters

Examples

# => ['awk', 'gawk', 'mawk' ...]

Returns the interpreters Array



423
424
425
# File 'lib/linguist/language.rb', line 423

def interpreters
  @interpreters
end

#language_idObject (readonly)

Public: Get language_id (used in GitHub search)

Examples

# => "1"
# => "2"
# => "3"

Returns the integer language_id



359
360
361
# File 'lib/linguist/language.rb', line 359

def language_id
  @language_id
end

#nameObject (readonly)

Public: Get proper name

Examples

# => "Ruby"
# => "Python"
# => "Perl"

Returns the name String



324
325
326
# File 'lib/linguist/language.rb', line 324

def name
  @name
end

#tm_scopeObject (readonly)

Public: Get the name of a TextMate-compatible scope

Returns the scope



364
365
366
# File 'lib/linguist/language.rb', line 364

def tm_scope
  @tm_scope
end

#typeObject (readonly)

Public: Get type.

Returns a type Symbol or nil.



333
334
335
# File 'lib/linguist/language.rb', line 333

def type
  @type
end

#wrapObject (readonly)

Public: Should language lines be wrapped

Returns true or false



405
406
407
# File 'lib/linguist/language.rb', line 405

def wrap
  @wrap
end

Class Method Details

.[](name) ⇒ Object

Public: Look up Language by its name.

name - The String name of the Language

Examples

Language['Ruby']
# => #<Language name="Ruby">

Language['ruby']
# => #<Language name="Ruby">

Returns the Language or nil if none was found.



223
224
225
226
227
228
229
230
# File 'lib/linguist/language.rb', line 223

def self.[](name)
  return nil if !name.is_a?(String) || name.to_s.empty?

  lang = @index[name.downcase]
  return lang if lang

  @index[name.split(',', 2).first.downcase]
end

.allObject

Public: Get all Languages

Returns an Array of Languages



104
105
106
# File 'lib/linguist/language.rb', line 104

def self.all
  @languages
end

.by_type(type) ⇒ Object

Detect languages by a specific type

type - A symbol that exists within TYPES

Returns an array



41
42
43
# File 'lib/linguist/language.rb', line 41

def self.by_type(type)
  all.select { |h| h.type == type }
end

.colorsObject

Public: A List of languages with assigned colors.

Returns an Array of Languages.



259
260
261
# File 'lib/linguist/language.rb', line 259

def self.colors
  @colors ||= all.select(&:color).sort_by { |lang| lang.name.downcase }
end

.create(attributes = {}) ⇒ Object

Internal: Create a new Language object

attributes - A hash of attributes

Returns a Language object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
# File 'lib/linguist/language.rb', line 50

def self.create(attributes = {})
  language = new(attributes)

  @languages << language

  # All Language names should be unique. Raise if there is a duplicate.
  if @name_index.key?(language.name)
    raise ArgumentError, "Duplicate language name: #{language.name}"
  end

  # Language name index
  @index[language.name.downcase] = @name_index[language.name.downcase] = language

  # Index filesystem name if defined
  if language.fs_name
    if @name_index.key?(language.fs_name)
      raise ArgumentError "Duplicate language name: #{language.fs_name}"
    end
    @index[language.fs_name.downcase] = @name_index[language.fs_name.downcase] = language
  end

  language.aliases.each do |name|
    # All Language aliases should be unique. Raise if there is a duplicate.
    if @alias_index.key?(name)
      raise ArgumentError, "Duplicate alias: #{name}"
    end

    @index[name.downcase] = @alias_index[name.downcase] = language
  end

  language.extensions.each do |extension|
    if extension !~ /^\./
      raise ArgumentError, "Extension is missing a '.': #{extension.inspect}"
    end

    @extension_index[extension.downcase] << language
  end

  language.interpreters.each do |interpreter|
    @interpreter_index[interpreter] << language
  end

  language.filenames.each do |filename|
    @filename_index[filename] << language
  end

  @language_id_index[language.language_id] = language

  language
end

.find_by_alias(name) ⇒ Object

Public: Look up Language by one of its aliases.

name - A String alias of the Language

Examples

Language.find_by_alias('cpp')
# => #<Language name="C++">

Returns the Language or nil if none was found.



133
134
135
136
# File 'lib/linguist/language.rb', line 133

def self.find_by_alias(name)
  return nil if !name.is_a?(String) || name.to_s.empty?
  name && (@alias_index[name.downcase] || @alias_index[name.split(',', 2).first.downcase])
end

.find_by_extension(filename) ⇒ Object

Public: Look up Languages by file extension.

The behaviour of this method recently changed. See the second example below.

filename - The path String.

Examples

Language.find_by_extension('dummy.rb')
# => [#<Language name="Ruby">]
Language.find_by_extension('rb')
# => []

Returns all matching Languages or [] if none were found.



173
174
175
176
177
178
179
180
# File 'lib/linguist/language.rb', line 173

def self.find_by_extension(filename)
  # find the first extension with language definitions
  extname = FileBlob.new(filename.downcase).extensions.detect do |e|
    !@extension_index[e].empty?
  end

  @extension_index[extname]
end

.find_by_filename(filename) ⇒ Object

Public: Look up Languages by filename.

The behaviour of this method recently changed. See the second example below.

filename - The path String.

Examples

Language.find_by_filename('Cakefile')
# => [#<Language name="CoffeeScript">]
Language.find_by_filename('foo.rb')
# => []

Returns all matching Languages or [] if none were found.



153
154
155
156
# File 'lib/linguist/language.rb', line 153

def self.find_by_filename(filename)
  basename = File.basename(filename)
  @filename_index[basename]
end

.find_by_id(language_id) ⇒ Object

Public: Look up Languages by its language_id.

language_id - Integer of language_id

Examples

Language.find_by_id(100)
# => [#<Language name="Elixir">]

Returns the matching Language



206
207
208
# File 'lib/linguist/language.rb', line 206

def self.find_by_id(language_id)
  @language_id_index[language_id.to_i]
end

.find_by_interpreter(interpreter) ⇒ Object

Public: Look up Languages by interpreter.

interpreter - String of interpreter name

Examples

Language.find_by_interpreter("bash")
# => [#<Language name="Bash">]

Returns the matching Language



192
193
194
# File 'lib/linguist/language.rb', line 192

def self.find_by_interpreter(interpreter)
  @interpreter_index[interpreter]
end

.find_by_name(name) ⇒ Object

Public: Look up Language by its proper name.

name - The String name of the Language

Examples

Language.find_by_name('Ruby')
# => #<Language name="Ruby">

Returns the Language or nil if none was found.



118
119
120
121
# File 'lib/linguist/language.rb', line 118

def self.find_by_name(name)
  return nil if !name.is_a?(String) || name.to_s.empty?
  name && (@name_index[name.downcase] || @name_index[name.split(',', 2).first.downcase])
end

Public: A List of popular languages

Popular languages are sorted to the top of language chooser dropdowns.

This list is configured in “popular.yml”.

Returns an Array of Languages.



240
241
242
# File 'lib/linguist/language.rb', line 240

def self.popular
  @popular ||= all.select(&:popular?).sort_by { |lang| lang.name.downcase }
end

.unpopularObject

Public: A List of non-popular languages

Unpopular languages appear below popular ones in language chooser dropdowns.

This list is created from all the languages not listed in “popular.yml”.

Returns an Array of Languages.



252
253
254
# File 'lib/linguist/language.rb', line 252

def self.unpopular
  @unpopular ||= all.select(&:unpopular?).sort_by { |lang| lang.name.downcase }
end

Instance Method Details

#==(other) ⇒ Object



481
482
483
# File 'lib/linguist/language.rb', line 481

def ==(other)
  eql?(other)
end

#default_aliasObject Also known as: default_alias_name

Public: Get default alias name

Returns the alias name String



450
451
452
# File 'lib/linguist/language.rb', line 450

def default_alias
  name.downcase.gsub(/\s/, '-')
end

#eql?(other) ⇒ Boolean

Returns:

  • (Boolean)


485
486
487
# File 'lib/linguist/language.rb', line 485

def eql?(other)
  equal?(other)
end

#escaped_nameObject

Public: Get URL escaped name.

Examples

"C%23"
"C%2B%2B"
"Common%20Lisp"

Returns the escaped String.



443
444
445
# File 'lib/linguist/language.rb', line 443

def escaped_name
  CGI.escape(name).gsub('+', '%20')
end

#get_typesObject



310
311
312
313
# File 'lib/linguist/language.rb', line 310

def get_types
  # Valid Languages types
  @types = [:data, :markup, :programming, :prose]
end

#groupObject

Public: Get Language group

Returns a Language



458
459
460
# File 'lib/linguist/language.rb', line 458

def group
  @group ||= Language.find_by_name(@group_name)
end

#hashObject



489
490
491
# File 'lib/linguist/language.rb', line 489

def hash
  name.hash
end

#inspectObject



493
494
495
# File 'lib/linguist/language.rb', line 493

def inspect
  "#<#{self.class} name=#{name}>"
end

#popular?Boolean

Public: Is it popular?

Returns true or false

Returns:

  • (Boolean)


465
466
467
# File 'lib/linguist/language.rb', line 465

def popular?
  @popular
end

#to_sObject

Public: Return name as String representation



477
478
479
# File 'lib/linguist/language.rb', line 477

def to_s
  name
end

#unpopular?Boolean

Public: Is it not popular?

Returns true or false

Returns:

  • (Boolean)


472
473
474
# File 'lib/linguist/language.rb', line 472

def unpopular?
  !popular?
end